import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory
    I1I2I3 : Otha's color space (1980)
    1st component I1 : intensity information   (R+G+B)/3         
    2nd component I2 : chromatic information   (R-B)/2
    3rd component I3 : chromatic information   (2G-R-B)/4
    I2 and I3 are designed in respect of decorrelation of RGB components
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p75-76
*/


public class fromRGBtoI1I2I3_ implements PlugInFilter {
	ImagePlus imp;
        public  int [] Pixels;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();			
		this.Pixels=new int[h*w];	
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	        IJ.showProgress(0.1);

	        String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));

		Pixels=(int[])ip.getPixels();
		byte[] I1=new byte[this.Pixels.length];
		byte[] I2=new byte[this.Pixels.length];
		byte[] I3=new byte[this.Pixels.length];

		this.getIII(I1,I2,I3);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);

		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_I1",I1);
		stack.addSlice(name+"_I2",I2);
		stack.addSlice(name+"_I3",I3);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    		imp2.show();

	}
       

         public void getIII(byte[] I1, byte[] I2, byte[] I3) {
	        int c,r,g,b;
		float[] III = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  r = (c&0xff0000)>>16;
		  g = (c&0xff00)>>8;
		  b = c&0xff;
		  III=RGBtoIII(r,g,b,III);
		  I1[i] = (byte)((int)III[0]);
		  I2[i] = (byte)((int)(III[1]+127.5f));
		  I3[i] = (byte)((int)(III[2]+127.5f));
		}
	 }


         public float[] RGBtoIII(int r, int g, int b, float[] IIIvals){
	       if (IIIvals==null)
    		  IIIvals =new float[3];
	       
	       IIIvals[0]=(r+g+b)/3.f;        // [0;255]
	       IIIvals[1]=(r-b)/2;            // [-127.5;127.5] 
	       IIIvals[2]=(2*g-r-b)/4;        // [-127.5;127.5] 
	       return IIIvals;
	 }

}


