import ij.*;
import ij.text.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 17/12/03 E. Glory 	
    
    This implementation gives  a float stack of L,u,v values no-normalized.  
   
    The Luv color space is uniform and recommended for applications in additive light source condition (CIE) 
    1st component L =              (luminance)
    2nd component u =              (a chromatic component )
    3rd component v =              (a chromatic component)
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p85-86
*/

public class fromRGBtoLuvFloat_ implements PlugInFilter {
	private ImagePlus 	imp;	
    private int [] 		Pixels;
    private	int 		illuminant;
    private	int 		height;
    private	int 		width;
    
    private TextWindow	tw;

	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		height=imp.getHeight();
		width=imp.getWidth();			
		this.Pixels=new int[height*width];	
		return DOES_RGB;
	}


	public void run(ImageProcessor ip) {
		Pixels=(int[])ip.getPixels();
		
		ImageStack stack = stackOpp(ip);	
		ImagePlus imp2=new ImagePlus(imp.getTitle(), new FloatProcessor(width, height));
		imp2.setStack(null, stack);
    	imp2.show();
    	imp2.updateAndDraw();
    	
    	// Normalisation en 8 bits
    	ImagePlus imp3 = new ImagePlus(imp.getTitle(), new FloatProcessor(width, height));
    	ImageStack stack2 = scaleStackTo8bits(stack);
		imp3.setStack(null, stack2);
    	imp3.show();
    	imp3.updateAndDraw();
    	
		//computeLimitsUV();	

	}


  	public ImageStack stackOpp(ImageProcessor ip) {
	    IJ.showProgress(0.1);

	    String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));

        float[] L=new float[this.Pixels.length];
		float[] u=new float[this.Pixels.length];
		float[] v=new float[this.Pixels.length];
		
		this.getLuv_float(L,u,v);
		//affiche(L,u,v,Pixels);

		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(width,height, cm);
		stack.addSlice(name+"_L_float",L);
		stack.addSlice(name+"_u_float",u);
		stack.addSlice(name+"_v_float",v);
		//System.out.println("nombre de slices "+stack.getSize());
		
    	IJ.showProgress(1.0);
    	return stack;
	}

	/* Luv non normalis : valeurs negatives et positives et possibilit >255*/
    public void getLuv_float(float[] L, float[] u, float[] v) {
	    int c,R,G,B;
		float[] Luv = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  
		  Luv=RGBtoLuv_float(R,G,B,Luv);
		  
		  L[i] = Luv[0];   
		  u[i] = Luv[1]; 
		  v[i] = Luv[2]; 

		}
	 }


     public float[] RGBtoLuv_float(int R, int G, int B, float[] Luvvals){
	       if (Luvvals==null)
    		  Luvvals =new float[3];
    		  
    	   // sRGB, illuminant D65
	       // ->The Color Image Processsing and Application, (plataniotis,Venetsanopoulos) p177)
	       // ->The Colour Image Processsing Handbook, (Sangwine,Horne) p71)
	       // for the EBU RGB space, the transformation in X,Y,Z is  

	       float  X = 0.412453f*R + 0.357580f*G + 0.180423f*B; // [0 ; 242.36628]
	       float  Y = 0.212671f*R + 0.715160f*G + 0.072169f*B; // [0 ; 255]
	       float  Z = 0.019334f*R + 0.119193f*G + 0.950227f*B; // [0 ; 277.63227]


	       // the D65 illuminant has X0, Y0, Z0 coordinates :	       
			float X0 = 0.950456f;
			float Y0 = 1.0f;
			float Z0 = 1.088754f;
	    
	    
	    
	    /**/

			
	       float  L = computeL(Y/Y0); 

	       float  u  = 4f*X /(X +15f*Y +3f*Z );
	       float  u0 = 4f*X0/(X0+15f*Y0+3f*Z0);
	       
	       float  v  = 9f*Y /(X +15f*Y +3f*Z );
 	       float  v0 = 9f*Y0/(X0+15f*Y0+3f*Z0);

	       Luvvals[0] = L;                  
	       Luvvals[1] = 13f*L*(u-u0);       
	       Luvvals[2] = 13f*L*(v-v0);   
	       
	       return Luvvals;
	 }

     public float computeL (float fY){
	   	   float result;
		   float epsilon = 216.f/24389.f;	
			
		   if(fY>epsilon)
		     result=(float)(116.*Math.pow(fY, 1./3.)-16);
		   else
		     result= (float)(24389.*fY/27.);

		   return result;
     } 
	
     public void affiche(byte[] L, byte[] u, byte[] v, int[] Pixels) {
	    TextWindow tw = new TextWindow("Control","img "+"level","",300,400); 
		for (int i=0; i<Pixels.length; i++){
			int l_=L[i]&0xff;
			int u_=u[i]&0xff;
			int v_=v[i]&0xff;
			tw.append("pixel "+i+" : Lab("+l_+","+u_+","+v_+")");
		}
	 }  
	 
	 public ImageStack scaleStackTo8bits(ImageStack stack){
		double value;
		double min;
		double max;
		double a;
		double b;
		
		ImageStack stackRes = new ImageStack(width,height);

		for (int i=1; i<=stack.getSize(); i++){
			ImageProcessor ipOri = stack.getProcessor(i);	
			min = ipOri.getMin();
			max = ipOri.getMax();
			a= 255./(max-min);
			b= (-min)*a;
			
			ByteProcessor bp = new ByteProcessor(width, height);
						
			for(int y=0; y<height; y++){
				for(int x=0; x<width; x++){
					value = ipOri.getPixelValue(x,y);
					bp.putPixelValue(x, y, (a*value)+b); 
				}	 	
			}
			stackRes.addSlice("normalized",bp); 
		 }
	 	 
	 	return stackRes;
	 }
	 

}


