import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory
    YUV is the European television color space used (Slater, 1991).
    1st component Y = 0.299 R + 0.587 G + 0.114 B         
    2nd component U =-0.147 R - 0.289 G + 0.437 B 
    3rd component V = 0.615 R - 0.515 G - 0.100 B
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p72
*/


public class fromRGBtoYUV_ implements PlugInFilter {
	ImagePlus imp;
        public  int [] Pixels;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();		
		this.Pixels=new int[h*w];		
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	        IJ.showProgress(0.1);

	        String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));
	
		Pixels=(int[])ip.getPixels();
		byte[] Y=new byte[this.Pixels.length];
		byte[] U=new byte[this.Pixels.length];
		byte[] V=new byte[this.Pixels.length];

		this.getYUV(Y,U,V);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);
		//imp2.trimProcessor();
		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_Y",Y);
		stack.addSlice(name+"_U",U);
		stack.addSlice(name+"_V",V);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    		imp2.show();

	}
       
         public void getYUV(byte[] Y, byte[] U, byte[] V) {
	        int c,R,G,B;
		float[] YUV = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  YUV=RGBtoYUV(R,G,B,YUV);
		  Y[i] = (byte)((int)(YUV[0]));
		  U[i] = (byte)((int)(YUV[1]*1.1445f+127.5f)); 
		  V[i] = (byte)((int)(YUV[2]*0.813f+127.5f));
		}
	 }

         public float[] RGBtoYUV(int R, int G, int B, float[] YUVvals){
	       if (YUVvals==null)
    		  YUVvals =new float[3];


	       YUVvals[0]= 0.299f*R+0.587f*G+0.114f*B;       // [0,255]
	       YUVvals[1]=-0.147f*R-0.289f*G+0.437f*B;       // [-111.4,111.4]
	       YUVvals[2]= 0.615f*R-0.515f*G-0.100f*B;       // [-156.825,156.825]
	       return YUVvals;
	 }
}
